package ch.psi.daq.queryrest.filter;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.filter.OncePerRequestFilter;

public class CorsFilter extends OncePerRequestFilter {

   private static final String ALLOW_ORIGIN_HEADER = "Access-Control-Allow-Origin";

   @Value("${queryrest.cors.allowedorigins}")
   private String configuredOrigins;
   
   @Value("${queryrest.cors.forceallheaders}")
   private boolean forceAllHeaders;
   


   /**
    * @{inheritDoc
    */
   @Override
   protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain)
         throws ServletException, IOException {

      Set<String> allowedOrigins = new HashSet<String>(Arrays.asList(configuredOrigins.split(","))
            .stream()
            .map(s -> {
               return s.trim(); })
            .collect(Collectors.toList()));
      
      String originHeader = request.getHeader("Origin");
      if (forceAllHeaders) {
         // include headers no matter what - good for development
         if (allowedOrigins.contains(originHeader)) {
            response.addHeader(ALLOW_ORIGIN_HEADER, originHeader);
         } else {
            response.addHeader(ALLOW_ORIGIN_HEADER, "*");
         }
         setDefaultCorsHeaders(response);
         
      } else if (request.getHeader("Access-Control-Request-Method") != null && "OPTIONS".equals(request.getMethod())) {
         // this is for 'real' Cross-site browser requests
         if (allowedOrigins.contains(originHeader)) {
            response.addHeader(ALLOW_ORIGIN_HEADER, originHeader);
         }
         setDefaultCorsHeaders(response);
      }

      filterChain.doFilter(request, response);
   }


   private void setDefaultCorsHeaders(HttpServletResponse response) {
      response.addHeader("Access-Control-Allow-Methods", "GET, POST, PUT, DELETE, OPTIONS");
      response.addHeader("Access-Control-Allow-Headers", "Origin, Authorization, Accept, Content-Type");
      response.addHeader("Access-Control-Max-Age", "1800");
   }


   public void setConfiguredOrigins(String configuredOrigins) {
      this.configuredOrigins = configuredOrigins;
   }


   public void setForceAllHeaders(boolean forceAllHeaders) {
      this.forceAllHeaders = forceAllHeaders;
   }
   
}
